/*****************************************************************************
**
**  kinematics.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <stdarg.h>


#include <uf.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>

#include "opu_kinematics.h"

#include "opu_kinematics_template.h"

#include "KinematicsMachines_utils.h"


#define COUNT_MACHINE 18

struct MACHINES
{
   int num;
   int methods;
   char name[30];
   int ai, bj, ck;
   char A[10], B[10], C[10];
   double minai, maxai;
   double minbj, maxbj;
   double minck, maxck;
} ;

static struct MACHINES machine_list[COUNT_MACHINE] =
{
 { 0,99,"Ce",1,1,0, "Theta","Phi","r",0,180,0,360,0,0}, //      
 { 1, 7,"MAG3ex",1,0,1,"A","B","C",-110,110,0,0,-360,360},
 { 2, 0,"Variaxis-630",1,0,1,"A","B","C",-120,30,0,0,-360,360},
 { 3, 1,"DMU-50ev",0,1,1,"A","B","C",0,0,0,180,-360,360},
 { 4, 0,"DMU-125PA",1,0,1,"A","B","C",-120,10,0,0,-360,360},
 { 5, 2,"DMU-125PB",0,1,1,"A","B","C",0,0,0,180,-360,360},
 { 6, 2,"DMU-200PT",0,1,1,"A","B","C",0,0,0,180,-360,360},
 { 7, 3,"V-STAR (+POGO)",0,1,1,"A","B","C",0,0,-110,110,-400,400},
 { 8, 3,"B-5BC",0,1,1,"A","B","C",0,0,-110,110,-360,360},
 { 9, 4,"FSQV-80KR",1,1,0,"A","B","C",-180,30,-90,90,0,0},
 {10, 4,"MA655c5",1,1,0,"A","B","C",-21.5,21.5,-21.5,21.5,0,0},
 {11, 4,">-147",1,1,0,"A","B","C",-21.5,21.5,-21.5,21.5,0,0},
 {12, 3,"NC-1325IP",0,1,1,"A","B","C",0,0,-105,105,-200,200},
 {13, 5,"P-6K",1,0,1,"A","B","C",-30,30,0,0,-270,270},
 {14, 6,"AGP630-800b",0,1,0,"A","B","C",0,0,-360,360,0,0},
 {15, 3,"DMU-35M",0,1,1,"A","B","C",0,0,-15,105,-360,360},
 {16, 8,"K. . Ipac",1,0,0, "A","B*","E*",-180,180,-12,12,-360,360},
 {17,-1,"none",0,0,0, "A","B","C",0,0,0,0,0,0}
};

/* This function read the numeric string after key letter string
  in success it return 1, otherwice 0
*/
int  read_str(char *s,char *w,char *sw)
{
   char *word,sym;
   int  n,m;  int fl_break;
   if(s==NULL) return(0);
   if ((word=strstr(s,w))==NULL) return(0);
   m=n=strlen(w);
   fl_break=0;
   do
   {  sym=*(word+n);
      if (isdigit(sym)) { fl_break++; n++; }
       else if (sym==' ' && fl_break!=0) break;
              else if (sym==' '||sym=='+'||sym=='-'||sym=='.') n++;
                    else break;
   } while (1);
   if (n>0){ n-=m; strncpy(sw,word+m,n); sw[n]='\0'; return(n); }
   else                                        return(0);
}

/* */
int delete_str(char *s,char *w)
{
 int n,m,i;
 char old_sw[50],*p;

 if(s==NULL) return (0);

 n=0;
 if ((n=read_str(s,w,old_sw))!=0)
  {
    p=strstr(s,w);
    if (p) {
     i=p-s;
     m=n+strlen(w);
     while(m!=0) {
       s[i]=' ';
       m--; i++ ;
     }

    }
  }

 return n;
}


int OutputMessage (int out, int err, char *format, ... )
{
   logical  response;
   va_list   ap;
   char      buffer[132];buffer[0]='\0';

   va_start( ap, format );
   vsprintf(buffer, format, ap );
   //vprintf( format, ap );
   va_end( ap );
   /*   */
   if (out==1) {
     UF_UI_is_listing_window_open (&response );
     if (response==FALSE) UF_UI_open_listing_window ( );
     UF_UI_write_listing_window ("\n");
     UF_UI_write_listing_window (buffer);
   } else {
     if (err!=0) { uc1601(buffer,1); }
   }

   return(0);
}


/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t KIN_cbs */
#define KIN_CB_COUNT ( 20 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with KIN_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t KIN_cbs[KIN_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, KIN_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_APPLY_CB        , 0, KIN_apply_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, KIN_cancel_cb},
 {KIN_OUTUG             , UF_STYLER_VALUE_CHANGED_CB, 0, KIN_outug},
 {KIN_ABOUT             , UF_STYLER_ACTIVATE_CB     , 0, KIN_about_cb},
 {KIN_LIST              , UF_STYLER_ACTIVATE_CB     , 0, KIN_list},
 {KIN_TYPE              , UF_STYLER_VALUE_CHANGED_CB, 0, KIN_type},
 {KIN_REAL_0            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_1            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_2            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_3            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_4            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_5            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_6            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_7            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_REAL_8            , UF_STYLER_ACTIVATE_CB     , 0, KIN_realvalue},
 {KIN_INSTR             , UF_STYLER_ACTIVATE_CB     , 0, KIN_instr},
 {KIN_FILETOG           , UF_STYLER_VALUE_CHANGED_CB, 0, KIN_file_tog},
 {KIN_FILEBUT           , UF_STYLER_ACTIVATE_CB     , 0, KIN_file_cb},
 {KIN_FILESTR           , UF_STYLER_ACTIVATE_CB     , 0, KIN_file_str},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};



/*****************************************************************************
**  Activation Methods
*****************************************************************************/
int loadDll( void )
{
    int  response   = 0;
    int errorCode;
    char *dialog_name="opu_kinematics.dlg\0";
    char env_names[][25]={
  	"UGII_USER_DIR" ,
  	"UGII_SITE_DIR" ,
  	"UGII_VENDOR_DIR" ,
  	"USER_UFUN" ,
  	"UGII_INITIAL_UFUN_DIR" ,
  	"UGII_TMP_DIR" ,
  	"HOME" ,
  	"TMP" } ;
 int i ;
 char *path , envpath[133] , dlgpath[255] , info[133];

  int module_id;
  UF_ask_application_module(&module_id);
  /*if (UF_APP_CAM!=module_id) {
     uc1601(" DLL -    CAM\n .",1);
     return(-1);
   }*/

 path = (char *) malloc(133+10);

 errorCode=-1;
  for (i=0;i<7;i++) {

    envpath[0]='\0';
    path=envpath;
    UF_translate_variable(env_names[i], &path);
    if (path!=NULL) {

       /*1*/
       dlgpath[0]='\0';
       strcpy(dlgpath,path); strcat(dlgpath,"\\application\\"); strcat(dlgpath,dialog_name);
       UF_print_syslog(dlgpath,FALSE);

       if ( ( errorCode = UF_STYLER_create_dialog ( dlgpath,
               KIN_cbs,      /* Callbacks from dialog */
               KIN_CB_COUNT, /* number of callbacks*/
               NULL,        /* This is your client data */
               &response ) ) != 0 )
        {
              /* Get the user function fail message based on the fail code.*/
              PrintErrorMessage( errorCode );
         }
       if (errorCode==0) break ;

       /*2*/
       dlgpath[0]='\0';
       strcpy(dlgpath,path); strcat(dlgpath,"\\"); strcat(dlgpath,dialog_name);
       UF_print_syslog(dlgpath,FALSE);

       if ( ( errorCode = UF_STYLER_create_dialog ( dlgpath,
               KIN_cbs,      /* Callbacks from dialog */
               KIN_CB_COUNT, /* number of callbacks*/
               NULL,        /* This is your client data */
               &response ) ) != 0 )
        {
              /* Get the user function fail message based on the fail code.*/
              PrintErrorMessage( errorCode );
         }
       if (errorCode==0) break ;

     } else { //if (envpath!=NULL)
     	info[0]='\0'; sprintf (info," %s -   \n ",env_names[i]);
     	UF_print_syslog(info,FALSE);
     }
  } // for

 if (errorCode<0) {
    info[0]='\0'; sprintf (info,"Don't load %s  \n ",dialog_name);
    uc1601 (info, TRUE );
  }

 return(errorCode);
}


int _construct_cb ( int dialog_id )
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i ;
    char *ls[COUNT_MACHINE];

     for (i=0;i<COUNT_MACHINE;i++)
     {
       ls[i]= (char *) malloc ((strlen(machine_list[i].name)+1)*sizeof(char)) ;
       ls[i][0]='\0';
       sprintf(ls[i],"%s",machine_list[i].name);

       UF_print_syslog(ls[i],FALSE);
     }

     data.item_id=KIN_LIST ;
     data.item_attr=UF_STYLER_SUBITEM_VALUES;
     data.value.strings=ls;
     data.count=COUNT_MACHINE;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

     data.item_attr=UF_STYLER_VALUE ;
     data.subitem_index=0;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

     UF_STYLER_free_value (&data) ;

     free(ls);

     return (0);
}


int _apply_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int outtog,Machine,Methods,filetog;
    char filename[132], szFilename1[132];
    int  status;
    FILE *rf;

    char Strmach[25];
    double ang[12], ijk[3], r ;
    long int i ;
    char mes[132] ;
    char stmp1[20], stmp2[20], stmp3[20], stmp11[20], stmp22[20], stmp33[20];

     data.item_attr=UF_STYLER_VALUE;

     data.item_id=KIN_LIST ;
     irc=UF_STYLER_ask_value(dialog_id,&data);   PrintErrorMessage( irc );
     Machine=data.value.integer;

     if (Machine<0) Machine=0;

     Strmach[0]='\0';  sprintf(Strmach,"%s>",machine_list[Machine].name);

     data.item_id=KIN_OUTUG;
     irc=UF_STYLER_ask_value(dialog_id,&data);   PrintErrorMessage( irc );
     outtog=data.value.integer;

     if (outtog==1) OutputMessage(outtog,0,"\n");

     data.item_id=KIN_TYPE;
     irc=UF_STYLER_ask_value(dialog_id,&data);   PrintErrorMessage( irc );
     Methods=data.value.integer;

     data.item_id=KIN_FILETOG;
     irc=UF_STYLER_ask_value(dialog_id,&data);   PrintErrorMessage( irc );
     filetog=data.value.integer;

    if (filetog) {
     	//  

        data.item_id=KIN_FILESTR;
        irc=UF_STYLER_ask_value(dialog_id,&data);  PrintErrorMessage( irc );
        filename[0]='\0'; sprintf(filename,"%s",data.value.string);
        UF_STYLER_free_value (&data) ;

        irc=UF_CFI_ask_file_exist (filename,&status );  PrintErrorMessage( irc );

        if (status!=0) {
          mes[0]='\0';  sprintf(mes,"  ->\n %s \n  ....",filename );
          uc1601(mes,1);
          return ( 0 );
        }

        szFilename1[0]='\0'; strcpy( szFilename1,filename); strcat( szFilename1,".log" ); // 2  set the file name

        if (Machine>=COUNT_MACHINE) return(0);
        i=machine_list[Machine].methods;
        if (i==99) i=-1;

        if (Methods==0) i=up_ABCijk(i,0,filename,szFilename1);
         else i=clsf_ijkABC(i,1,filename,szFilename1);

        mes[0]='\0';
        if (i<0) sprintf(mes,"   .\n  =%d \n  ....",i);
        else sprintf(mes,"  ->\n %s \n  ... \n    =%d ",szFilename1,i);
        uc1601(mes,1);
        if (i<0) { return ( 0 ); }

        if (outtog==1) {

          /* */
          rf=fopen(szFilename1,"r");
          if (rf==NULL) {
            mes[0]='\0';  sprintf(mes,"  :\n %s \n  '' \n",szFilename1 );
            uc1601(mes,1);
            return ( 0 );
          }

          UF_UI_open_listing_window();

          UF_UI_write_listing_window("\n-----------------------------------------------");
          mes[0]='\0';  sprintf(mes,"\n: %s ",szFilename1 );
          UF_UI_write_listing_window(mes);
          mes[0]='\0';  sprintf(mes,"\n: %s ",Strmach );
          UF_UI_write_listing_window(mes);
          mes[0]='\0';
          if (Methods==0) sprintf(mes,"\n : (A,B,C)->(i,j,k) ");
          else sprintf(mes,"\n : (i,j,k)->(A,B,C) " );
          UF_UI_write_listing_window(mes);
          UF_UI_write_listing_window("\n-----------------------------------------------\n");

          mes[0]='\0'; fgets(mes,132,rf);
          while(!feof(rf))
          {
            /**/
            UF_UI_write_listing_window(mes);

            mes[0]='\0'; fgets(mes,132,rf);
          } // end while
          fclose(rf);
          UF_UI_write_listing_window(mes);

        }

        return ( 0 );

    }

 //    + //  
  for(i=0;i<12;i++) ang[i]=0.;
  for(i=0;i<3;i++) ijk[i]=0.; ijk[2]=1.;
 //    .
  switch (Methods)
  {
   case 0 :  // A,B,C->i,j,k

        data.item_id=KIN_REAL_0;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ang[0]=data.value.real;

        data.item_id=KIN_REAL_1;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ang[1]=data.value.real;

        data.item_id=KIN_REAL_2;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ang[2]=data.value.real;

   break ;
   default :

        data.item_id=KIN_REAL_6;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ijk[0]=data.value.real;

        data.item_id=KIN_REAL_7;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ijk[1]=data.value.real;

        data.item_id=KIN_REAL_8;
        irc=UF_STYLER_ask_value(dialog_id,&data);        PrintErrorMessage( irc );
        ijk[2]=data.value.real;


       r=sqrt(ijk[0]*ijk[0]+ijk[1]*ijk[1]+ijk[2]*ijk[2]);
       if (EQ_is_zero(r)) {
       	 ijk[0]=0.; ijk[1]=0.; ijk[2]=1.; r=1.;
         //OutputMessage(outtog,0,"  :");
       	 OutputMessage(outtog,0,"\t!! ={i,j,k} - .");
       }
       if (!EQ_is_equal(r,1.000)) {
            //OutputMessage(outtog,0,"  :");
            OutputMessage(outtog,0,"\t !! ={i,j,k} -  ->... koef=%.3lf ",1./r);
            ijk[0]/= r; ijk[1]/= r; ijk[2]/= r;
       }

        data.item_id=KIN_REAL_6;
        data.value.real=ijk[0];
        irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

        data.item_id=KIN_REAL_7;
        data.value.real=ijk[1];
        irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

        data.item_id=KIN_REAL_8;
        data.value.real=ijk[2];
        irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  }



  if (machine_list[Machine].ai) {
    if (ang[0]<machine_list[Machine].minai || ang[0]>machine_list[Machine].maxai)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].A,machine_list[Machine].minai,machine_list[Machine].maxai);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }
  if (machine_list[Machine].bj) {
    if (ang[1]<machine_list[Machine].minbj || ang[1]>machine_list[Machine].maxbj)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].B,machine_list[Machine].minbj,machine_list[Machine].maxbj);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }
  if (machine_list[Machine].ck) {
    if (ang[2]<machine_list[Machine].minck || ang[2]>machine_list[Machine].maxck)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].C,machine_list[Machine].minck,machine_list[Machine].maxck);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }

  //OutputMessage(Strmach + "-----------------------------in-----------------------------");

  // 
  switch (machine_list[Machine].num)
  {

   case 0 : // :   -> (Theta,Phi)<->(i,j,k)
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],180.);
          ang[1]=CheckConst(ang[1],360.);
        break ;
        default : ; break ;
        }
        i=ABCijk_sphera(Methods,ang,ijk);
   break;

   case 1 : //MAG3  A,C
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],360.);
          ang[2]=CheckConst(ang[2],360.);
        break ;
        default : ; break ;
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   case 2 : //Variaxis 630-5x  A(),C()
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],360.);
          ang[2]=CheckConst(ang[2],360.);
        break ;
        default : ; break ;
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   case 3 :    // DMU-50eV  B(),C()
       switch (Methods)
       {
        case 0 :
          ang[1]=CheckConst(ang[1],360.);
          ang[2]=CheckConst(ang[2],360.);
          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
        break ;
        default :
          if (ijk[2]<0.) { ijk[2]=fabs(ijk[2]); OutputMessage(outtog,-1,"%s ! k<0 - ! - ....",Strmach); }

          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);

          OutputMessage(outtog,0,"%s .  1: B3=%.4lf  3=%.4lf ",Strmach,ang[7],ang[8]);
          OutputMessage(outtog,0,"%s .  2: B4=%.4lf  4=%.4lf ",Strmach,ang[10],ang[11]);
        }
   break;

   case 4 :    // DMU-125PA  A(),C()
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],360.);
          ang[2]=CheckConst(ang[2],360.);
        break ;
        default :
          if (EQ_is_zero(ijk[2])) { ijk[2]=1.; OutputMessage(outtog,-1,"%s ! k=0 - ! - ....",Strmach); }
          //if (ijk[2]<0.) { ijk[2]=fabs(ijk[2]); OutputMessage(outtog,-1,"%s ! k<0 - ! - ....",Strmach); }
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   // DMU-125P  DMU-200P  B(),C()
   case 5 : case 6 :
       switch (Methods)
       {
        case 0 :
          ang[1]=CheckConst(ang[1],360.);
          ang[2]=CheckConst(ang[2],360.);
          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
        break ;
        default :
          //if (EQ_is_zero(ijk[2])) { ijk[2]=1.; OutputMessage(outtog,-1,"%s ! k=0 - ! - ....",Strmach); }
          if (ijk[2]<0.) { ijk[2]=fabs(ijk[2]);  OutputMessage(outtog,-1,"%s ! k<0 - ! - ....",Strmach); }

          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);

          //ang[1]=CheckConst(ang[1],360.);
          //ang[2]=CheckConst(ang[2],360.);

          //  
          OutputMessage(outtog,0,"%s .  1: B3=%.4lf  3=%.4lf ",Strmach,ang[7],ang[8]);
          OutputMessage(outtog,0,"%s .  2: B4=%.4lf  4=%.4lf ",Strmach,ang[10],ang[11]);
        }
   break;


   // V-STAR v3500ORB- B,C(); // -5 B(=A),C- // NC-1325IP  B(),C()// DMU-35M -  B=-15..+105 , C=360
   case 7 : case 8 : case 12 : case 15 :
       switch (Methods)
       {
        case 0 :
          ang[1]=CheckConst(ang[1],360.);
          if (machine_list[Machine].num==7) {
            if (ang[2]>400.) { ang[2]-=360. ; }
            if (ang[2]<-400.) { ang[2]+=360. ; }
          } else {
            ang[2]=CheckConst(ang[2],360.);
          }
          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
        break ;
        default :
          i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
          if (machine_list[Machine].num==12) {
            if (ang[2]<-200.) ang[2]+=360.;
            if (ang[2]>200.) ang[2]-=360.;
          }
        break ;
        }
   break;

   // FSQV80KR -  A,B (); // MA655c5 ,14V7  -  A,B ()
   case 9 : case 10 : case 11 :
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],360.);
          ang[1]=CheckConst(ang[1],360.);
        break ;
        default :
         if (machine_list[Machine].num!=9) {
          if (EQ_is_zero(ijk[2])) { ijk[2]=1.; OutputMessage(outtog,-1,"%s ! k=0 - ! - ....",Strmach); }
          if (ijk[2]<0.) { ijk[2]=fabs(ijk[2]); OutputMessage(outtog,-1,"%s ! k<0 - ! - ....",Strmach); }
         }
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
        OutputMessage(outtog,0,"%s  (i,j,k) <-> (A,B)  .  ",Strmach);
   break;

   case 13 :    // RFP-6K  A(),C()
       switch (Methods)
       {
        case 0 :
          ang[0]=CheckConst(ang[0],360.);
          ang[2]=CheckConst(ang[2],360.);
        break ;
        default :
          if (EQ_is_zero(ijk[2])) { ijk[2]=1.; OutputMessage(outtog,-1,"%s ! k=0 - ! - ....",Strmach); }
          if (ijk[2]<0.) { ijk[2]=fabs(ijk[2]); OutputMessage(outtog,-1,"%s ! k<0 - ! - ....",Strmach); }
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   case 14 :    // -630-800  B()
       switch (Methods)
       {
        case 0 :
           ang[1]=CheckConst(ang[1],360.);
        break ;
        default :
          if (!EQ_is_zero(ijk[1])) { OutputMessage(outtog,-1,"%s !   j=0 ",Strmach); }
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   case 16 :    // K >
       switch (Methods)
       {
        case 0 :
           ang[0]=CheckConst(ang[0],360.);
        break ;
        default :
           if (!EQ_is_zero(ijk[0])) { OutputMessage(outtog,-1,"%s !   i=0 ",Strmach); }
        break ;
        }
        i=ABCijk(machine_list[Machine].methods,Methods,ang,ijk);
   break;

   case 17 : // none
       switch (Methods)
       {
        case 0 :
        break ;
        default : ; break ;
        }
   break;

   default :
            uc1601("  ",1);
  }



  if (machine_list[Machine].ai) {
    if (ang[0]<machine_list[Machine].minai || ang[0]>machine_list[Machine].maxai)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].A,machine_list[Machine].minai,machine_list[Machine].maxai);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }
  if (machine_list[Machine].bj) {
    if (ang[1]<machine_list[Machine].minbj || ang[1]>machine_list[Machine].maxbj)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].B,machine_list[Machine].minbj,machine_list[Machine].maxbj);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }
  if (machine_list[Machine].ck) {
    if (ang[2]<machine_list[Machine].minck || ang[2]>machine_list[Machine].maxck)
    {
      mes[0]='\0'; sprintf(mes,"! %s %.1lf..%.1lf",machine_list[Machine].C,machine_list[Machine].minck,machine_list[Machine].maxck);
      OutputMessage(outtog,-1,"%s %s ",Strmach,mes);
    }
  }

// 

  data.item_id=KIN_REAL_0;
  data.value.real=ang[0];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  data.item_id=KIN_REAL_1;
  data.value.real=ang[1];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  data.item_id=KIN_REAL_2;
  data.value.real=ang[2];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  stmp1[0]='\0'; stmp2[0]='\0'; stmp3[0]='\0';
  stmp11[0]='\0'; stmp22[0]='\0'; stmp33[0]='\0';
  switch (Methods)
  {
   case 0 :  // A,B,C->i,j,k

     if (machine_list[Machine].ai) {
         sprintf(stmp1,"%s%.5lf",machine_list[Machine].A,ang[0]);
     }
     if (machine_list[Machine].bj) {
         sprintf(stmp2,"%s%.5lf",machine_list[Machine].B,ang[1]);
     }
     if (machine_list[Machine].ck) {
         sprintf(stmp3,"%s%.5lf",machine_list[Machine].C,ang[2]);
     }
     OutputMessage(outtog,0,"%s (%s %s %s) -> ( %.8lf , %.8lf , %.8lf )",Strmach,stmp1,stmp2,stmp3,ijk[0],ijk[1],ijk[2]);

   break ;
   default : // i,j,k->A,B,C

     data.item_id=KIN_REAL_3;
     data.value.real=ang[3];
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_4;
     data.value.real=ang[4];
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_5;
     data.value.real=ang[5];
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     if (machine_list[Machine].ai) {
         sprintf(stmp1,"%s%.5lf",machine_list[Machine].A,ang[0]);
         sprintf(stmp11,"%s%.5lf",machine_list[Machine].A,ang[3]);
     }
     if (machine_list[Machine].bj) {
         sprintf(stmp2,"%s%.5lf",machine_list[Machine].B,ang[1]);
         sprintf(stmp22,"%s%.5lf",machine_list[Machine].B,ang[4]);
     }
     if (machine_list[Machine].ck) {
      	if (EQ_is_zero(ijk[0]) && EQ_is_zero(ijk[1]) && !EQ_is_zero(ijk[2])) {
      	  sprintf(stmp3,"%s= ",machine_list[Machine].C);
      	  sprintf(stmp33,"%s= ",machine_list[Machine].C);
      	} else {
      	  sprintf(stmp3,"%s%.5lf",machine_list[Machine].C,ang[2]);
      	  sprintf(stmp33,"%s%.5lf",machine_list[Machine].C,ang[5]);
      	}
     }
    OutputMessage(outtog,0,"%s ( %.8lf , %.8lf , %.8lf ) -> ( %s  %s  %s )  ( %s  %s  %s )",Strmach,ijk[0],ijk[1],ijk[2],stmp1,stmp2,stmp3,stmp11,stmp22,stmp33);

  }

  data.item_id=KIN_REAL_6;
  data.value.real=ijk[0];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  data.item_id=KIN_REAL_7;
  data.value.real=ijk[1];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  data.item_id=KIN_REAL_8;
  data.value.real=ijk[2];
  irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  UF_STYLER_free_value (&data) ;

  return (0);
}


int _about_cb ( int dialog_id)
{
   char *messages[]={
   	 " :; .  : ",
   	 " :",
   	 "  1)  ;",
   	 "  2)   (A,B,C)<->(i,j,k);",
   	 "  3)      ;",
   	 "  4)   'Apply'",
   	 "  ",
   	 NULL };
   UF_UI_message_buttons_t buttons = { TRUE, FALSE, FALSE, "", "", "", 1, 0, 0 };
   int response;
   UF_UI_message_dialog ("  ...",UF_UI_MESSAGE_INFORMATION,messages,7,0,&buttons,&response ) ;
   return(0);
}


int _list ( int dialog_id )
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i;

     data.item_id=KIN_LIST ;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
     i=data.value.integer;

     if (i<0) i=0;

     data.item_id=KIN_REAL_0;
     data.item_attr=UF_STYLER_SENSITIVITY;
     data.value.integer=machine_list[i].ai;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );
     data.item_attr=UF_STYLER_LABEL;
     data.value.string=machine_list[i].A;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

     data.item_id=KIN_REAL_1;
     data.item_attr=UF_STYLER_SENSITIVITY;
     data.value.integer=machine_list[i].bj;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );
     data.item_attr=UF_STYLER_LABEL;
     data.value.string=machine_list[i].B;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

     data.item_id=KIN_REAL_2;
     data.item_attr=UF_STYLER_SENSITIVITY;
     data.value.integer=machine_list[i].ck;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );
     data.item_attr=UF_STYLER_LABEL;
     data.value.string=machine_list[i].C;
     irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

     UF_STYLER_free_value (&data) ;

    return (0);
}


int _type ( int dialog_id)
{
   UF_STYLER_item_value_type_t data  ;
   int irc ;
   int type;

   data.item_attr=UF_STYLER_VALUE;
   data.item_id=KIN_TYPE;
   irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
   type=data.value.integer;

   data.item_attr=UF_STYLER_LABEL;
   data.item_id=KIN_FILETOG;
   if (type) data.value.string=": (CLSF)  .\0";
    else data.value.string=": ()  .\0";
   irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

   data.item_attr=UF_STYLER_SENSITIVITY ;

   data.item_id=KIN_REAL_3;
   data.value.integer=type;
   irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

   data.item_id=KIN_REAL_4;
   data.value.integer=type;
   irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

   data.item_id=KIN_REAL_5;
   data.value.integer=type;
   irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

   UF_STYLER_free_value (&data) ;

   return (0);

}


int _instr ( int dialog_id )
{
  UF_STYLER_item_value_type_t data  ;
  int irc ;
  int index_mach,index_meth;
  int index;
  double i,j,k;
  double r ;
  char s[256];
  char word[20];
  int cnt[3];
  double ai,aj,ak;


     /* ---- Enter your callback code here ----- */
     data.item_attr=UF_STYLER_VALUE;

     data.item_id=KIN_LIST ;
     irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
     index_mach=data.value.integer;

     data.item_id=KIN_TYPE;
     irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
     index_meth=data.value.integer;

     data.item_id=KIN_INSTR;
     irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
     s[0]='\0';sprintf(s,"%s",data.value.string);

  cnt[0]=-1; cnt[1]=-1; cnt[2]=-1; //   
     i=0.;      j=0. ;     k=0. ;
    ai=0.;     aj=0. ;    ak=0. ;

  switch (index_meth)
  {
   case 0 :  // A,B,C->i,j,k

     if (index_mach>=COUNT_MACHINE) return(0) ;

     index=0;
     while (s[index]!='\0')
     {
       if (s[index]=='=' || s[index]==';' || s[index]==',' || \
             s[index]=='(' || s[index]==')' || s[index]=='|' ) s[index]=' ';
       index++;
     }

     if (machine_list[index_mach].ai) {
      if (read_str(s,machine_list[index_mach].A,word))
      {
        if (1==sscanf(word,"%lf",&ai)) {
          /*   */ delete_str(s,machine_list[index_mach].A);
          cnt[0]=1;
        }
      }
     }
     if (machine_list[index_mach].bj) {
      if (read_str(s,machine_list[index_mach].B,word))
      {
        if (1==sscanf(word,"%lf",&aj)) {
          /*   */ delete_str(s,machine_list[index_mach].B);
          cnt[1]=1;
        }
      }
     }
     if (machine_list[index_mach].ck) {
      if (read_str(s,machine_list[index_mach].C,word))
      {
        if (1==sscanf(word,"%lf",&ak)) {
          /*   */ delete_str(s,machine_list[index_mach].C);
          cnt[2]=1;
        }
      }
     }

   break ;
   default : ; // i,j,k->A,B,C
  }


  index=0;
  while (s[index]!='\0')
  {
    if (s[index]=='=' || s[index]==';' || s[index]==',' || \
          s[index]=='(' || s[index]==')' || s[index]=='|' || \
            isalpha(s[index]) ) s[index]=' ';
    index++;
  }

  index=sscanf(s,"%lf %lf %lf",&i,&j,&k);

  switch(index_meth)
  {
  	case 0: // A,B,C->i,j,k

     if (machine_list[index_mach].ai>0 && cnt[0]>0) { i=ai; }
     if (machine_list[index_mach].bj>0 && cnt[1]>0) { j=aj; }
     if (machine_list[index_mach].ck>0 && cnt[2]>0) { k=ak; }

     data.item_id=KIN_REAL_0;
     data.value.real=i;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_1;
     data.value.real=j;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_2;
     data.value.real=k;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  	break;
  	default: // i,j,k->A,B,C

     r=sqrt(i*i+j*j+k*k);
     if (EQ_is_zero(r)) {
          i=0.; j=0.; k=1.; r=1.;
     }
     if (!EQ_is_equal(r,1.000)) {
          s[0]='\0'; sprintf(s," : !! ={i,j,k} -  ->... koef=%.3lf ",1./r);
          i/= r; j/= r; k/= r;
     }

     data.item_id=KIN_REAL_6;
     data.value.real=i;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_7;
     data.value.real=j;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

     data.item_id=KIN_REAL_8;
     data.value.real=k;
     irc=UF_STYLER_set_value(dialog_id,&data);        PrintErrorMessage( irc );

  	break;
  }

  UF_STYLER_free_value (&data) ;

  return (0);

}


int _file_tog ( int dialog_id )
{
  UF_STYLER_item_value_type_t data  ;
  int irc ;
  int type;

  data.item_attr=UF_STYLER_VALUE;
  data.item_id=KIN_FILETOG;
  irc=UF_STYLER_ask_value(dialog_id,&data);    PrintErrorMessage( irc );
  type=data.value.integer;

  data.item_attr=UF_STYLER_SENSITIVITY ;

  data.item_id=KIN_FILEBUT;
  data.value.integer=type;
  irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

  data.item_id=KIN_FILESTR;
  data.value.integer=type;
  irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

  data.item_attr=UF_STYLER_VISIBILITY ;

  data.item_id=KIN_RC_BEG_VALUE;
  data.value.integer=!type;
  irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

  data.item_id=KIN_INSTR;
  data.value.integer=!type;
  irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

  UF_STYLER_free_value (&data) ;

  return ( 0 );

}


int _file_cb ( int dialog_id )
{
  UF_STYLER_item_value_type_t data  ;
  int irc ;
  char title_string[132],prompt_string[132],filter_string[132];
  char filename[132];
  int response;
  int type;

  data.item_attr=UF_STYLER_VALUE;
  data.item_id=KIN_TYPE;
  irc=UF_STYLER_ask_value(dialog_id,&data);     PrintErrorMessage( irc );
  type=data.value.integer;

  title_string[0]='\0';
  prompt_string[0]='\0';
  filter_string[0]='\0';
  switch(type)
  {
  	case 0:
  	sprintf(filter_string,"*.*"); //*.up,|*.ptp|*.*
  	sprintf(prompt_string," .....");
  	sprintf(title_string," ");
  	break;
  	default:
  	sprintf(filter_string,"*.cls"); //|*.apt|*.*
  	sprintf(prompt_string," CLSF.....");
  	sprintf(title_string," CLSF");
  	break;
  }

  filename[0]='\0';
  irc = UF_UI_create_filebox (prompt_string,
                             title_string,
                             filter_string,
                             NULL,
                             filename,
                             &response);
  PrintErrorMessage( irc );
  if (response==UF_UI_OK) {

   data.item_id=KIN_FILESTR;
   data.value.string=filename;
   irc=UF_STYLER_set_value(dialog_id,&data);     PrintErrorMessage( irc );

  }

  UF_STYLER_free_value (&data) ;

  return (0);
}


int _realvalue ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
  UF_STYLER_item_value_type_t data  ;


  UF_STYLER_free_value (&data) ;
	return (0);
}


/*  Explicit Activation
**      This entry point is used to activate the application explicitly, as in
**      "File->Execute UG/Open->User Function..." */
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
        UF_print_syslog("ufusr>start (kinematics)",FALSE);

        loadDll();

        UF_print_syslog("ufusr->end (kinematics)",FALSE);
        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode=0;
}
/*****************************************************************************
**  Utilities
*****************************************************************************/

/* Unload Handler
**     This function specifies when to unload your application from Unigraphics.
**     If your application registers a callback (from a MenuScript item or a
**     User Defined Object for example), this function MUST return
**     "UF_UNLOAD_UG_TERMINATE". */
extern int ufusr_ask_unload( void )
{
     /* unload immediately after application exits*/
     return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     //return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}


/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}


/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        fprintf( stderr, "%s\n", message );
    }
}


/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: KIN_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _construct_cb (dialog_id);
     KIN_type (dialog_id,client_data,callback_data);
     KIN_file_tog (dialog_id,client_data,callback_data);
     KIN_list (dialog_id,client_data,callback_data);

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_apply_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_apply_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
	/* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
    _apply_cb ( dialog_id,client_data,callback_data);

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog                 */
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must respond to your apply button.*/
    return (UF_UI_CB_CONTINUE_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_outug
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_outug ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_about_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_about_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _about_cb ( dialog_id );

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_list
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_list ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _list ( dialog_id ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_type
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_type ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _type ( dialog_id ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_realvalue
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_realvalue ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _realvalue (dialog_id,client_data,callback_data);
     KIN_apply_cb (dialog_id,client_data,callback_data);

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_instr
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_instr ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _instr (dialog_id);
     KIN_apply_cb (dialog_id,client_data,callback_data);

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_file_tog
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_file_tog ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _file_tog ( dialog_id ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_file_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_file_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _file_cb ( dialog_id );

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: KIN_file_str
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int KIN_file_str ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}
